@@ -17,6 +17,7 @@
 #![deny(clippy::all)]
 
 /* simplified assertion */
+/// Exits with error message if condition is true
 #[macro_export]
 macro_rules! die {
     ($exit:expr, $exit_code:expr, $($arg:tt)*) => {
@@ -32,6 +33,14 @@ macro_rules! die {
     };
 }
 
+/// Returns product of all elements in a list
+#[macro_export]
+macro_rules! product {
+    ($x:expr) => {
+        $x.iter().fold(1, |acc, x| acc * x)
+    };
+}
+
 /// Returns a random boolean value.
 pub fn rand_bool() -> bool {
     use fastrand::bool as fastbool;
@@ -59,12 +68,27 @@ pub fn is_executable(cmd: &str) -> bool {
     Command::new(cmd).output().is_ok()
 }
 
+/// Raises base to the power of exp and returns the result.
+pub fn power(base: u128, exp: u128) -> u128 {
+    base.pow(exp.try_into().unwrap())
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
 
     #[test]
-    fn test_rand_bool() {
+    fn test_product_vec() {
+        assert_eq!(product!(vec![1, 2, 3, 4]), 24);
+    }
+
+    #[test]
+    fn product_slice() {
+        assert_eq!(product!([1, 2, 3, 4]), 24);
+    }
+
+    #[test]
+    fn rand_bool_obvious() {
         let rand_bool: bool = rand_bool();
         assert_eq!(rand_bool, rand_bool);
     }
@@ -84,7 +108,7 @@ mod tests {
     }
 
     #[test]
-    fn test_is_even_any() {
+    fn is_even_any_100() {
         for n in 0..100 {
             if is_even(n) {
                 assert!(is_even_any(&n));
@@ -95,7 +119,7 @@ mod tests {
     }
 
     #[test]
-    fn test_any_to_string() {
+    fn any_to_string_all() {
         assert_eq!(any_to_string(0), "0".to_string());
         assert_eq!(any_to_string("0".to_string()), "0".to_string());
         assert_eq!(any_to_string(0u8), "0".to_string());
@@ -115,8 +139,21 @@ mod tests {
     }
 
     #[test]
-    fn test_is_executable() {
+    fn is_executable_cmds() {
         assert!(is_executable("ls"));
         assert!(!is_executable("lsd"));
     }
+
+    #[test]
+    fn power_mem() {
+        assert_eq!(power(2, 3), 8);
+        assert_eq!(power(2, 4), 16);
+        assert_eq!(power(2, 5), 32);
+        assert_eq!(power(2, 6), 64);
+        assert_eq!(power(2, 7), 128);
+        assert_eq!(power(2, 8), 256);
+        assert_eq!(power(2, 9), 512);
+        assert_eq!(power(2, 10), 1024);
+        assert_eq!(power(2, 11), 2048);
+    }
 }
